
/*  -*- MaTX -*-
 *
 *  NAME
 *      quad8() - Numerical evaluation of integrals (quadrature)
 *
 *  SYNOPSIS
 *      {q,cnt} = quad8(funfcn,a,b)
 *           Real q;
 *           Integer cnt;
 *           Matrix funfcn();
 *           Real a,b;
 *                     
 *      {q,cnt} = quad8(funfcn,a,b,tol)
 *           Real q;
 *           Integer cnt;
 *           Matrix funfcn();
 *           Real a,b;
 *           Real tol;
 *                     
 *      {q,cnt} = quad8(funfcn,a,b,tol,trc)
 *           Real q;
 *           Integer cnt;
 *           Matrix funfcn();
 *           Real a,b;
 *           Real tol;
 *           Integer trc;
 *                     
 *  DESCRIPTION
 *      quad8() approximates the integral of funfcn(x) from a to b to
 *      within a relative error of 1e-3.  funfcn(x) must return {q,cnt}
 *      a vector of output values if given a vector of input values.
 *
 *      quad8(...,tol) integrates to a relative error of tol.
 *
 *      q = Inf is returned if an excessive recursion level is reached,
 *      indicating a possibly singular integral.
 *
 *      quad(...,trc) integrates to a relative error of tol and
 *      traces the function evaluations with a point plot of the integrand.
 *
 *      quad8() uses an adaptive recursive Newton Cotes 8 panel rule.
 *      quad8() also returns a function evaluation count.
 *
 *  SEE ALSO
 *      quadf().
 */

Func List quad8(funfcn,a,b,tol,trc, ...)
	Matrix funfcn();
	Real a,b,tol;
	Integer trc;
{
	Integer cnt,win,lev,midx,nidx;
	Matrix lims,w,x,y;
	Real q,h;
	Index idx;

	List quad8stp();

	error(nargchk(3, 5, nargs, "quad8"));

	win = mgplot_cur_win();

	if (nargs < 4) {
		tol = 1.0e-3;
		trc = 0;
	}
	if (nargs < 5) {
		trc = 0;
	}

	// quad8 usually does better than the default 1e-3.
	h = b - a;

	// Top level initialization, Newton-Cotes weights
	w = [3956, 23552, -3712, 41984, -18160, 41984, -3712, 23552, 3956]/14175;

	x = a .+ [0:8]*(b-a)/8;
	y = funfcn(x);

	lims = [min(x) max(x) min(y) max(y)];
	idx = find( ! isfinite(lims));

	if ( ! isempty(idx)) {
		midx = length(idx);
		nidx = 1;
		lims(idx) = 1.0e30 * (-ONE(midx,nidx) .^ rem(Array(idx),2));
	}

	if (trc) {
		mgplot(win, [a b], [y(1) y(9)]);
	}
	lev = 1;

	// Adaptive, recursive Newton-Cotes 8 panel quadrature
	{q,cnt} = quad8stp(funfcn,a,b,tol,lev,w,x,y,2.0^1023,trc);
//	{q,cnt} = quad8stp(funfcn,a,b,tol,lev,w,x,y,Inf,trc);
	cnt = cnt + 9;

	return {q,cnt};
}


/*  -*- MaTX -*-
 *
 *  NAME
 *      quad8stp() - Recursive function used by quad8()
 *
 *  SYNOPSIS
 *      {q,cnt} = quad8stp(funfcn,a,b,tol,lev,w,x0,f0,q0,trc)
 *           Real q;
 *           Integer cnt;
 *           Matrix funfcn();
 *           Real a,b,tol;
 *           Integer lev;
 *           Matrix w,x0,f0;
 *           Real q0;
 *           Integer trc;
 *                
 *  DESCRIPTION
 *      Recursive function driven by quad8().
 *      quad8stp() tries to approximate the integral of funfcn(x)
 *      from a to b to within a relative error of tol.  
 *      The arguments are generated by quad8 or by the recursion.
 *
 *      lev : recursion level
 *        w : weights in the 8 panel Newton Cotes formula
 *       x0 : Vector of 9 equally spaced abscissa
 *       f0 : Vector of the 9 function values at x
 *       q0 : Approximate value of the integral
 *
 *  SEE ALSO
 *      quad8() and quad().
 *
 */

Func List quad8stp(funfcn,a,b,tol,lev,w,x0,f0,q0,trc)
	Matrix funfcn();
	Real a,b,tol;
	Integer lev;
	Matrix w,x0,f0;
	Real q0;
	Integer trc;
{
	Integer lev_max,cnt,cnt1,cnt2;
#ifndef VC40
	Integer win;
#endif
	Matrix f,x;
	Real h,c,q,q1,q2;

	lev_max = 10;
#ifndef VC40
	win = mgplot_cur_win();
#endif

	// Evaluate function at midpoints of left and right half intervals
	x = Z(1,17);
	f = Z(1,17);
	x(1:2:17) = x0;
	f(1:2:17) = f0;
	x(2:2:16) = (x0(1:8) + x0(2:9))/2;
	f(2:2:16) = funfcn(x(2:2:16));

#ifndef VC40
	if (trc) {
		mgplot(win, x(2:2:16), f(2:2:16));
	}
#endif
	cnt = 8;

	// Integrate over half intervals
	h = (b - a)/16;
	q1 = [h*w*f(1:9)#](1);
	q2 = [h*w*f(9:17)#](1);
	q = q1 + q2;

	// Recursively refine approximations
	if (abs(q - q0) > tol*abs(q) && lev <= lev_max) {
		c = (a + b)/2;
		{q1,cnt1} = quad8stp(funfcn,a,c,tol/2,lev+1,w,x(1:9),f(1:9),q1,trc);
		{q2,cnt2} = quad8stp(funfcn,c,b,tol/2,lev+1,w,x(9:17),f(9:17),q2,trc);
		q = q1 + q2;
		cnt = cnt + cnt1 + cnt2;
	}

	return {q,cnt};
}
