
/*  -*- MaTX -*-
 *
 *  NAME
 *      rat() - Rational approximation
 *
 *  SYNOPSIS
 *      {S,X} = rat(X0)
 *          Matrix S, X;
 *          Matrix X0;
 *
 *      {S,X} = rat(X0,str)
 *          Matrix S, X;
 *          Matrix X0;
 *          String str;
 *
 *      {S,X} = rat(X0,str,len)
 *          Matrix S, X;
 *          Matrix X0;
 *          String str;
 *          Integer len;
 *
 *      {S,X} = rat(X0,str,len,dmax)
 *          Matrix S, X;
 *          Matrix X0;
 *          String str;
 *          Integer len;
 *          Integer dmax;
 *
 *  DESCRIPTION
 *      rat() returns a rational approximation to the elements of X by
 *      removing the roundoff error from results that should be "simple"
 *      rational numbers.
 *	
 *      rat() returns two integer matrices N and D such that N/D = rat(X).
 *      rat(X,"s") returns a text matrix with the rational approximation to X. 
 *
 *      For example, try rat(hilb(4)).
 *
 *      The rational approximations are generated by truncating continued 
 *      fraction expansions.  
 *
 *      rat(X,str,len,dmax) controls the length of the expansion and the
 *      truncation level.
 *
 */

Func List rat(X_,str,len,dmax, ...)
	Matrix X_;
	String str;
	Integer len,dmax;
{
	Matrix s,S,X;
	Integer decrat,den,k,m,n,i,j,t;
	Index d;
	Real x,num,p;
	String sj;
	List SS, ss;
	
	error(nargchk(1, 4, nargs, "rat"));

	X = X_;

	decrat = 0;	// do rational approximation into text matrix
	if (nargs < 2) {
		len = 13;
		dmax = 10000;
		decrat = 1;
	} else if (nargs == 2) {
		len = 13;
		dmax = 10000;
		if (length(str) == 0) {
			decrat = 1;
		} else {
			decrat = 0;
		}
	} else if (nargs == 3) {
		dmax = 10000;
		if (length(str) == 0) {
			decrat = 1;
		} else {
			decrat = 0;
		}
	}

	{m,n} = size(X);
	if (decrat) {
		S = Z(m,n);
	} else {
		SS = makelist(m);
	}
	if (decrat) {
		s = Z(1,n);
	} else {
		ss = makelist(n);
	}
	for (i = 1; i <= m; i++) {
		for (j = 1; j <= n; j++) {
			x = abs(X(i,j));
			d = [fix(x)];
			x = x - d(1);
			k = 1;
			p = 1.0/abs(dmax);
			while (x > p) {
				x = 1/x;
				k = k + 1;
				d = [d, [fix(x)]];
				x = x - d(k);
				p = p*d(k);
			}
			if (dmax < 0) {
				print d;
			}
			num = d(k);
			den = 1;
			for (k = k-1; k >= 1; k--) {
				t = Integer(num);
				num = d(k)*num + den;
				den = t;
			}
			num = sgn(X(i,j))*num;

			if (decrat == 0) {
				sj = sprintf("%d", Integer(num));
				if (den != 1) {
					sj = sj + "/" + sprintf("%d",den);
				} else {
					sj = " " + sj;
				}

				if (length(sj) > len || (num == 0 && x != 0)) {
					sj = sprintf("%13.3e",X(i,j));
				} else {
					sj = sj + " " * (len-length(sj));
				}
				ss(j) = sj;
			} else {
				s(j) = num;
			}
			X(i,j) = den;
		}

		if (decrat) {
			S(i,:) = s;
		} else {
			SS(i) = ss;
		}
	}

	if (decrat == 1) {
		S = S./X;
		return {S,X};
	} else {
		return {SS,X};
	}
}
