
/*  -*- MaTX -*-
 *
 *  NAME
 *      decimate() - Resample data at a lower rate after lowpass filtering
 *
 *  SYNOPSIS
 *      Y = decimate(X,R,nfilt)
 *         Array Y;
 *         Array X;
 *         Integer R, nfilt;
 *
 *      Y = decimate(X,R,nfilt,option)
 *         Array Y;
 *         Array X;
 *         Integer R, nfilt;
 *         String option;
 *
 *  DESCRIPTION
 *      decimate() resamples the sequence in vector X at 1/R
 *      times the original sample rate.  The resulting resampled
 *      vector Y is R times shorter, length(Y) = length(X)/R.
 *
 *      decimate() filters the data with an eighth order Chebyshev
 *      type I lowpass filter with cutoff frequency 0.8*(Fs/2)/R,
 *      before resampling.
 *
 *      decimate(X,R,N) uses an N-th order Chebyshev filter.
 *      decimate(..., "FIR") uses the 30 point FIR filter
 *      generated by fir1(30,1/R) to filter the data.
 *      decimate(..., "FIR") uses the N-point FIR filter.
 *
 *  SEE ALSO
 *      interp().
 *
 */

Func Array decimate(idata,r,nfilt,option, ...)
	Array idata;
	Integer r;
	Integer nfilt;
	String option;
{
	Matrix a,b;
	Array odata,gd,itemp;
	Integer fopt,lod,m,n,nbeg,nd,nout,nlen,zi,zf;
	Real rip;
	Index lst;

	error(nargchk(2, 4, nargs, "decimate"));

	if (r == 1) {
		odata = idata;
		return odata;
	}
	if (r <= 0) {
		error("decimate(): Resampling rate R must be a positive integer.\n");
	}

	fopt = 1;
	if (nargs == 2) {
		nfilt = 8;
	} else if (nargs == 4) {
		if (option(1) == "f" || option(1) == "F") {
			fopt = 0;
		} else if (option(1) == "i" || option(1) == "I") {
			fopt = 1;
		} else {
			error("decimate(): Filter options are FIR and IIR.\n");
		}
	}
	if (fopt == 1 && nfilt > 13) {
		warning("decimate(): IIR filters above order 13 may be unreliable.\n");
	}

	nd = max(Cols(idata), Rows(idata));
	nout = Integer(ceil(nd/Real(r)));
	{m,n} = size(idata);

	// FIR filter
	if (fopt == 0) { 
		b = fir1(nfilt,1/Real(r));

		// Prepend sequence with mirror image of first points so that
		// transients can be eliminated. then do same thing at right
		// end of data sequence.
		nfilt = nfilt + 1;
		itemp = 2*idata(1) .- idata((nfilt+1):-1:2);
		{itemp,zi} = filter(b,[1],itemp);
		{odata,zf} = filter(b,[1],idata,zi);

		if (m == 1) { // row data
			itemp = Z(1,2*nfilt);
		} else {      // column data
			itemp = Z(2*nfilt,1);
		}

		itemp = reshape(2*idata(nd) .- idata((nd-1):-1:(nd-2*nfilt)),
						Rows(itemp), Cols(itemp));
		{itemp} = filter(b,[1],itemp,zf);

		// finally, select only every r'th point from the interior of
		// the lowpass filtered sequence
		{gd} = grpdelay(b,1,8);
		lst = [round(gd(1)+1.25):r:nd];
		odata = odata(lst);
		lod = length(odata);
		nlen = nout - lod;
		nbeg = Integer(r - (nd - lst(length(lst))));

		if (m == 1) { // row data
			odata = [odata, itemp(nbeg:r:nbeg+nlen*r-1)];
		} else {
			odata = [[odata][itemp(nbeg:r:nbeg+nlen*r-1)]];
		}
	} else {
		// IIR filter
		rip = 0.05;  // passband ripple in dB
		{b,a} = cheby1(nfilt, rip, [0.8/r]);

		// Be sure to filter in both directions to make sure the filtered
		// data has zero phase make a data vector properly pre- and ap- 
		// pended to filter forwards and back so end effects can be 
		// obliterated.
		odata = filtfilt(b,a,idata);
		nbeg = r - (r*nout - nd);
		odata = odata(nbeg:r:nd);
	}
	return odata;
}

